# encoding: utf8

SAMPLE_RATE = 16000
SPEECH_LENGTH = 30

NOTIME_SYMBOL = "<notimestamp>"
FIRST_TIME_SYMBOL = "<0.00>"
LAST_TIME_SYMBOL = "<30.00>"
FIRST_LANG_SYMBOL = "<ab>"
LAST_LANG_SYMBOL = "<zu>"
FIRST_REGION_SYMBOL = "<AD>"
LAST_REGION_SYMBOL = "<ZA>"
